package be.iminds.ilabt.jfed.espec.util;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.List;

public class UploadProgressTrackerImpl implements UploadProgressTracker {
    private final List<UploadProgressTrackerImpl.ChangeListener> listeners = new ArrayList();

    private int current;
    private int total; //-1 -> unknown

    public UploadProgressTrackerImpl() {
        this.current = 0;
        this.total = -1;
    }
    public UploadProgressTrackerImpl(int total) {
        this.current = 0;
        this.total = total;
    }

    @Override
    public void addChangeListener(@Nonnull UploadProgressTrackerImpl.ChangeListener l) {
        listeners.add(l);
    }

    @Override
    public void removeChangeListener(@Nonnull UploadProgressTrackerImpl.ChangeListener l) {
        listeners.remove(l);
    }


    @Override
    public void setCurrentBytes(int current) {
        boolean changed = this.current != current;
        if (changed) {
            this.current = current;
            fireChange();
        }
    }

    @Override
    public void setTotalBytes(int total) {
        boolean changed = this.total != total;
        if (changed) {
            this.total = total;
            fireChange();
        }
    }

    private void fireChange() {
        for (UploadProgressTrackerImpl.ChangeListener l : listeners)
            l.onChanged(this);
    }

    @Override
    public int getCurrentBytes() {
        return current;
    }

    @Override
    public int getTotalBytes() {
        return total;
    }

    @Override
    public double getPercentProgress() {
        return (current * 100.0) / total;
    }

    @Override
    public boolean isTotalBytesKnown() {
        return total >= 0;
    }

    @Override
    public boolean isFinished() {
        return total >= 0 && current >= total;
    }
}
